@@ -1,6 +1,7 @@
 # Author: Babak Naimi, naimi.b@gmail.com
 # Date :  July 2012
-# Version 1.0
+# Last Update :  Oct. 2021
+# Version 1.1
 # Licence GPL v3
 
 if (!isGeneric("endpoints")) {
@@ -14,3 +15,9 @@ setMethod("endpoints", "RasterStackBrickTS",
           }
           )
 
+
+setMethod("endpoints", "SpatRasterTS",
+          function(x, on="months", k=1) {
+            endpoints(x@time,on = on, k = k)
+          }
+)
\ No newline at end of file
